@@ -30,10 +30,9 @@
 #include "bool.h"
 #include "mallocvar.h"
 
-#include "xmlrpc-c/base.h"
-#include "xmlrpc-c/base_int.h"
-#include "xmlrpc-c/client.h"
+#include "xmlrpc-c/string_int.h"
 #include "xmlrpc-c/client_int.h"
+#include "xmlrpc-c/transport.h"
 
 /* The libwww interface */
 
@@ -54,8 +53,6 @@
 #include "WWWSSL.h"
 #endif
 
-#include "xmlrpc_libwww_transport.h"
-
 /* This value was discovered by Rick Blair. His efforts shaved two seconds
 ** off of every request processed. Many thanks. */
 #define SMALLEST_LEGAL_LIBWWW_TIMEOUT (21)
@@ -87,10 +84,10 @@ typedef struct {
     int http_status;
 
     /* Low-level information used by libwww. */
-    HTRequest *request;
-    HTChunk *response_data;
-    HTParentAnchor *source_anchor;
-    HTAnchor *dest_anchor;
+    HTRequest *      request;
+    HTChunk *        response_data;
+    HTParentAnchor * source_anchor;
+    HTAnchor *       dest_anchor;
 
     xmlrpc_transport_asynch_complete complete;
     struct xmlrpc_call_info * callInfoP; 
@@ -120,8 +117,11 @@ initLibwww(const char * const appname,
            const char * const appversion) {
     
     /* We initialize the library using a robot profile, because we don't
-    ** care about redirects or HTTP authentication, and we want to
-    ** reduce our application footprint as much as possible. */
+       care about redirects or HTTP authentication, and we want to
+       reduce our application footprint as much as possible.
+       
+       This takes the place of HTLibInit().
+    */
     HTProfile_newRobot(appname, appversion);
 
     /* Ilya Goldberg <igg@mit.edu> provided the following code to access
@@ -169,11 +169,16 @@ create(xmlrpc_env *                      const envP,
        int                               const flags,
        const char *                      const appname,
        const char *                      const appversion,
-       const struct xmlrpc_xportparms *  const transportParmsP ATTR_UNUSED,
+       const void *                      const transportParmsP ATTR_UNUSED,
        size_t                            const parm_size ATTR_UNUSED,
        struct xmlrpc_client_transport ** const handlePP) {
 /*----------------------------------------------------------------------------
    This does the 'create' operation for a Libwww client transport.
+
+   TODO: put 'appname' and 'appversion' in *transportParmsP and
+   deprecate the create() arguments.  Reason: these are particular to
+   the Libwww transport.  They're create() arguments because originally,
+   Libwww was the only transport.
 -----------------------------------------------------------------------------*/
     /* The Libwww transport is not re-entrant -- you can have only one
        per program instance.  Even if we changed the Xmlrpc-c code not
@@ -222,8 +227,11 @@ destroy(struct xmlrpc_client_transport * const clientTransportP) {
 
     if (!(clientTransportP->saved_flags & XMLRPC_CLIENT_SKIP_LIBWWW_INIT)) {
         HTProfile_delete();
+            /* This takes the place of HTLibTerminate() */
     }
     destroyCookieJar(clientTransportP->cookieJarP);
+    HTConversion_deleteAll(clientTransportP->xmlrpc_conversions);
+    HTProfile_delete();
 }
 
 
@@ -232,6 +240,44 @@ destroy(struct xmlrpc_client_transport * const clientTransportP) {
 **  HTTP Error Reporting
 **=======================================================================*/
 
+static void
+formatLibwwwError(HTRequest *   const requestP,
+                  const char ** const msgP) {
+/*----------------------------------------------------------------------------
+   When something fails in a Libwww request, Libwww generates a stack
+   of error information (precious little information, of course, in the
+   Unix tradition) and attaches it to the request object.  We make a message
+   out of that information.
+
+   We rely on Libwww's HTDialog_errorMessage() to do the bulk of the
+   formatting; we might be able to coax more information out of the request
+   if we interpreted the error stack directly.
+-----------------------------------------------------------------------------*/
+    HTList * const errStack = HTRequest_error(requestP);
+    
+    if (errStack == NULL)
+        xmlrpc_asprintf(msgP, "Libwww supplied no error details");
+    else {
+        /* Get an error message from libwww.  The middle three
+           parameters to HTDialog_errorMessage appear to be ignored.
+           XXX - The documentation for this API is terrible, so we may
+           be using it incorrectly.  
+        */
+        const char * const msg =
+            HTDialog_errorMessage(requestP, HT_A_MESSAGE, HT_MSG_NULL,
+                                  "An error occurred", errStack);
+        
+        if (msg == NULL)
+            xmlrpc_asprintf(msgP, "Libwww supplied some error detail, "
+                            "but its HTDialog_errorMessage() subroutine "
+                            "mysteriously failed to interpret it for us.");
+        else
+            *msgP = msg;
+    }
+}
+
+
+
 static void 
 set_fault_from_http_request(xmlrpc_env * const envP,
                             int          const status,
@@ -245,45 +291,20 @@ set_fault_from_http_request(xmlrpc_env * const envP,
     if (status == 200) {
         /* No error.  Don't set one in *envP */
     } else {
-        /* Get an error message from libwww. The middle three
-           parameters to HTDialog_errorMessage appear to be ignored.
-           XXX - The documentation for this API is terrible, so we may
-           be using it incorrectly.  
-        */
-        HTList * const errStack = HTRequest_error(requestP);
-    
-        if (errStack == NULL) {
-            /* I think this is probably impossible, because we didn't get
-               status 200, but I don't completely understand HTTP and libwww.
-            */
+        const char * libwwwMsg;
+        formatLibwwwError(requestP, &libwwwMsg);
+
+        if (status == -1)
             xmlrpc_env_set_fault_formatted(
                 envP, XMLRPC_NETWORK_ERROR,
-                "HTTP error #%d occurred, but there was no additional "
-                "error information supplied", status);
-        } else {
-            const char * const msg = 
-                HTDialog_errorMessage(requestP, HT_A_MESSAGE, HT_MSG_NULL,
-                                      "An error occurred", errStack);
-
-            if (msg == NULL)
-                xmlrpc_env_set_fault_formatted(
-                    envP, XMLRPC_NETWORK_ERROR,
-                    "HTTP error #%d occurred.  Libwww's "
-                    "HTDialog_errorMessage() routine was mysteriously "
-                    "unable to interpret the additional error information, "
-                    "so we have none to report.", status);
-            else {
-                /* Set our fault.  Note that this may inlcude line breaks,
-                   because 'msg' may.  We should fix that.  Formatting for
-                   display is none of our business at this level.
-                */
-                xmlrpc_env_set_fault_formatted(
-                    envP, XMLRPC_NETWORK_ERROR,
-                    "HTTP error #%d occurred.  libwww says %s", status, msg);
-                
-                xmlrpc_strfree(msg);
-            }
+                "Unable to complete the HTTP request.  %s", libwwwMsg);
+        else {
+            xmlrpc_env_set_fault_formatted(
+                envP, XMLRPC_NETWORK_ERROR,
+                "HTTP request completed with HTTp error %d.  %s",
+                status, libwwwMsg);
         }
+        xmlrpc_strfree(libwwwMsg);
     }
 }
 
@@ -401,7 +422,7 @@ createDestAnchor(xmlrpc_env *               const envP,
                  HTAnchor **                const destAnchorPP,
                  const xmlrpc_server_info * const serverP) {
                  
-    *destAnchorPP = HTAnchor_findAddress(serverP->_server_url);
+    *destAnchorPP = HTAnchor_findAddress(serverP->serverUrl);
 
     if (*destAnchorPP == NULL)
         xmlrpc_env_set_fault_formatted(
@@ -461,9 +482,9 @@ rpcCreate(xmlrpc_env *                       const envP,
     HTRequest_setRqHd(rpcP->request, request_headers);
 
     /* Send an authorization header if we need one. */
-    if (serverP->_http_basic_auth)
+    if (serverP->allowedAuth.basic)
         HTRequest_addCredentials(rpcP->request, "Authorization",
-                                 serverP->_http_basic_auth);
+                                 (char *)serverP->basicAuthHdrValue);
     
     /* Make sure there is no XML conversion handler to steal our data.
     ** The 'override' parameter is currently ignored by libwww, so our
@@ -510,8 +531,8 @@ static void
 rpcDestroy(rpc * const rpcP) {
 
     XMLRPC_ASSERT_PTR_OK(rpcP);
-    XMLRPC_ASSERT(rpcP->request != XMLRPC_BAD_POINTER);
-    XMLRPC_ASSERT(rpcP->response_data != XMLRPC_BAD_POINTER);
+    XMLRPC_ASSERT(rpcP->request != NULL);
+    XMLRPC_ASSERT(rpcP->response_data != NULL);
 
     /* Junji Kanemaru reports on 05.04.11 that with asynch calls, he
        get a segfault, and reversing the order of deleting the request
@@ -525,9 +546,9 @@ rpcDestroy(rpc * const rpcP) {
     */
 
     HTRequest_delete(rpcP->request);
-    rpcP->request = XMLRPC_BAD_POINTER;
+    rpcP->request = NULL;
     HTChunk_delete(rpcP->response_data);
-    rpcP->response_data = XMLRPC_BAD_POINTER;
+    rpcP->response_data = NULL;
 
     /* This anchor points to private data, so we're allowed to delete it.  */
     deleteSourceAnchor(rpcP->source_anchor);
@@ -558,6 +579,16 @@ extract_response_chunk(xmlrpc_env *        const envP,
                        rpc *               const rpcP,
                        xmlrpc_mem_block ** const responseXmlPP) {
 
+    /* Implementation warning: A Libwww chunk has nothing to do with
+       an HTTP chunk.  HTTP chunks (as in a chunked response) are not
+       visible to us; Libwww delivers the entire unchunked body to us
+       at once (we never see chunk headers and trailers).  This
+       subroutine is about a Libwww chunk, which is just a memory
+       buffer.  (Libwww is capable of delivering the response in a
+       number of ways other than a chunk, e.g. it can write it to a
+       file.
+    */
+
     /* Check to make sure that w3c-libwww actually sent us some data.
     ** XXX - This may happen if libwww is shut down prematurely, believe it
     ** or not--we'll get a 200 OK and no data. Gag me with a bogus design
@@ -642,7 +673,7 @@ call(xmlrpc_env *                     const envP,
                           rpcP->request);
         if (!ok)
             xmlrpc_env_set_fault(
-                envP, XMLRPC_INTERNAL_ERROR,
+                envP, XMLRPC_NETWORK_ERROR,
                 "Libwww HTPostAnchor() failed to start POST request");
         else {
             /* Run our event-processing loop.  HTEventList_newLoop()
@@ -686,7 +717,7 @@ static int timer_called = 0;
 static void 
 register_asynch_call(void) {
     XMLRPC_ASSERT(outstanding_asynch_calls >= 0);
-    outstanding_asynch_calls++;
+    ++outstanding_asynch_calls;
 }
 
 
@@ -695,7 +726,7 @@ static void
 unregister_asynch_call(void) {
 
     XMLRPC_ASSERT(outstanding_asynch_calls > 0);
-    outstanding_asynch_calls--;
+    --outstanding_asynch_calls;
     if (outstanding_asynch_calls == 0)
         HTEventList_stopLoop();
 }
@@ -703,9 +734,9 @@ unregister_asynch_call(void) {
 
 
 static int 
-timer_callback(HTTimer *   const timer ATTR_UNUSED,
+timer_callback(HTTimer *   const timer     ATTR_UNUSED,
                void *      const user_data ATTR_UNUSED,
-               HTEventType const event) {
+               HTEventType const event     ATTR_UNUSED) {
 /*----------------------------------------------------------------------------
   A handy timer callback which cancels the running event loop. 
 -----------------------------------------------------------------------------*/
@@ -901,7 +932,7 @@ sendRequest(xmlrpc_env *                     const envP,
                           rpcP->request);
         if (!ok) {
             unregister_asynch_call();
-            xmlrpc_env_set_fault(envP, XMLRPC_INTERNAL_ERROR,
+            xmlrpc_env_set_fault(envP, XMLRPC_NETWORK_ERROR,
                                  "Libwww (HTPostAnchor()) failed to start the "
                                  "POST request.");
         }
@@ -913,9 +944,12 @@ sendRequest(xmlrpc_env *                     const envP,
 
 
 struct xmlrpc_client_transport_ops xmlrpc_libwww_transport_ops = {
+    NULL,
+    NULL,
     &create,
     &destroy,
     &sendRequest,
     &call,
     &finishAsynch,
+    NULL,
 };
